# Generated by Django 2.0.13 on 2019-03-06 08:39

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [("address", "0006_auto_20181115_1953")]

    operations = [
        migrations.CreateModel(
            name="PaymentMethod",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("label", models.CharField(max_length=32)),
                ("key", models.CharField(max_length=32)),
                (
                    "country",
                    models.ManyToManyField(
                        to="address.Country", verbose_name="Countries"
                    ),
                ),
            ],
        )
    ]
