import doctest

import ocyan.plugin.oscar_checkout.utils


def load_tests(loader, tests, ignore):  # pylint: disable=W0613
    tests.addTests(doctest.DocTestSuite(ocyan.plugin.oscar_checkout.utils))
    return tests
