@media screen and (max-width: 991px){
  .try-again-form button{
      position: fixed;
      bottom: 0;
      z-index: 99999;
      margin: 0 !important;
      width: 50% !important;
  }
  .try-again-form .btn-warning{
      left: 0;
  }
  .try-again-form .btn-primary{
      right: 0;  
  }  
}
