#billing-address-form-block {
  display: none;
}
#shipping-method-list,
#id_payment_method_form-payment_method {
  list-style-type: none;
  list-style-position: inside;
  padding: 5px 10px;
}
.basket-totals-panel {
  width: 100%;
  list-style-type: none;
  list-style-position: inside;
  padding: 0;
}

.basket-totals-panel > .basket-line.row {
  width: 100%;
  margin: 0;
}
.basket-totals-panel > .basket-line.row .product-image {
  padding: 0;
}

.basket-totals-panel > .basket-line.row .product-specification {
  padding: 0;
}
.basket-totals-panel > .basket-line.row img.thumbnail {
  margin: 0;
}
.basket-totals-panel > .basket-line.row > div > .total {
  background-color: white !important;
  padding: 0px 0px;
}

.basket-totals-panel > .basket-line.row > div > .item-description {
  padding: 0px 10px;
}

#gateway > div.panel-body > #passwords {
  max-height: 0;
  overflow: hidden;
  -webkit-transition: max-height 0.2s; /* For Safari 3.1 to 6.0 */
  transition: max-height 0.2s;
  margin-top: 0px;
}
#loginbutton {
  display: none;
}

#gateway > div.panel-body > div label {
  margin: 0;
}
#gateway > div.panel-body > div {
  margin: 10px 0 0 0;
}
#gateway > div.panel-body > div ul {
  padding: 7px 0 0 7px;
  margin-bottom: 7px;
}
#gateway > div.panel-body > div li {
  list-style: none;
  padding-left: 0;
}

span.payment-image[name*="paypal"] {
  background: url(/static/img/paypal.png) no-repeat;
  background-size: 100%;
  width: 28px;
  height: 24.217284px;
  display: inline-block;
  background-position: left;
}

span.payment-image[name*="ideal"] {
  background: url(/static/img/IDEAL_Logo.png) no-repeat;
  background-size: 100%;
  width: 28px;
  height: 24.217284px;
  display: inline-block;
  background-position: left;
}

span.payment-image[name*="wallet"] {
  background: url(/static/img/logomsp.png) no-repeat;
  background-size: 100%;
  width: 37px;
  height: 24.217284px;
  display: inline-block;
  background-position: left;
}

span.payment-image[name*="creditcard"] {
  background: url(/static/img/creditcard.jpg) no-repeat;
  background-size: 100%;
  width: 31px;
  height: 24.217284px;
  display: inline-block;
  background-position: left;
}

span.payment-image[name*="mistercash"], span.payment-image[name*="bancontact"] {
  background: url(/static/img/bankcontact.png) no-repeat;
  background-size: 100%;
  width: 30px;
  height: 24.217284px;
  display: inline-block;
  background-position: left;
}

span.payment-image[name*="banktrans"],
span.payment-image[name*="overboeking"] {
  background: url(/static/img/Overboeking.png) no-repeat;
  background-size: 100%;
  width: 28px;
  height: 24.217284px;
  display: inline-block;
  background-position: left;
}

span.payment-image[name*="directdebit"] {
  background: url(/static/img/SEPA-incasso-logo.png) no-repeat;
  background-size: 100%;
  width: 29px;
  height: 24.217284px;
  display: inline-block;
  background-position: left;
}

span.payment-image[name*="sofort"],
span.payment-image[name*="directbank"] {
  background: url(/static/img/sofort.png) no-repeat;
  background-size: 100%;
  width: 32px;
  height: 24.217284px;
  display: inline-block;
  background-position: left;
}

span.payment-image[name*="eps"] {
  background: url(/static/img/eps.png) no-repeat;
  background-size: 100%;
  width: 28px;
  height: 24.217284px;
  display: inline-block;
  background-position: left;
}

span.payment-image[name*="kbc"],
span.payment-image[name*="cbc"] {
  background: url(/static/img/kbccbc.png) no-repeat;
  background-size: 100%;
  width: 28px;
  height: 24.217284px;
  display: inline-block;
  background-position: left;
}

span.payment-image[name*="homepay"],
span.payment-image[name*="inghome"] {
  background: url(/static/img/home-pay.jpg) no-repeat;
  background-size: 100%;
  width: 38px;
  height: 24.217284px;
  display: inline-block;
  background-position: left;
}

span.payment-image[name*="giropay"] {
  background: url(/static/img/Giropay.png) no-repeat;
  background-size: 100%;
  width: 28px;
  height: 24.217284px;
  display: inline-block;
  background-position: left;
}

span.payment-image[name*="giftcard"] {
  background: url(/static/img/kadeaukaart.png) no-repeat;
  background-size: 100%;
  width: 28px;
  height: 24.217284px;
  display: inline-block;
  background-position: left;
}

span.payment-image[name*="belfius"] {
  background: url(/static/img/BelfiusLogoNeutral.png) no-repeat;
  background-size: 100%;
  width: 33px;
  height: 24.217284px;
  display: inline-block;
  background-position: left;
}

span.payment-image[name*="bitcoin"] {
  background: url(/static/img/bitcoin.png) no-repeat;
  background-size: 100%;
  width: 28px;
  height: 24.217284px;
  display: inline-block;
  background-position: left;
}

span.payment-image[name*="mastercard"] {
  background: url(/static/img/MasterCard_Logo.svg.png) no-repeat;
  background-size: 100%;
  width: 28px;
  height: 24.217284px;
  display: inline-block;
  background-position: left;
}

span.payment-image[name*="trustly"] {
  background: url(/static/img/trustly-logo.png) no-repeat;
  background-size: 100%;
  width: 31px;
  height: 24.217284px;
  display: inline-block;
  background-position: left;
}

span.payment-image[name*="visa"] {
  background: url(/static/img/ojf8ed4taaxccncp6pcp.jpg) no-repeat;
  background-size: 100%;
  width: 30px;
  height: 24.217284px;
  display: inline-block;
  background-position: left;
}

span.payment-image[name*="maestro"] {
  background: url(/static/img/maestro_creditcard.jpg) no-repeat;
  background-size: 100%;
  width: 28px;
  height: 24.217284px;
  display: inline-block;
  background-position: left;
}

span.payment-image[name*="dummy:payment"] {
  background: url(/static/img/dummypaymentlogo.png) no-repeat;
  background-size: 100%;
  width: 28px;
  height: 30px;
  display: inline-block;
  background-position: left;
}

span.payment-image[name*="ebill"] {
  background: url(/static/img/ebill.png) no-repeat;
  background-size: 100%;
  width: 37px;
  height: 50.217284px;
  display: inline-block;
  background-position: left;
}

span.payment-image[name*="billink"] {
  background: url(/static/img/billink.png) no-repeat;
  background-size: 100%;
  width: 30px;
  height: 24.217284px;
  display: inline-block;
  background-position: left;
}

span.payment-image[name*="bunq"] {
  background: url(/static/img/Bunq.png) no-repeat;
  background-size: 100%;
  width: 30px;
  height: 50.217284px;
  display: inline-block;
  background-position: left;
}

span.payment-image[name*="vpay"] {
  background: url(/static/img/VPay.png) no-repeat;
  background-size: 100%;
  width: 23px;
  height: 50.217284px;
  display: inline-block;
  background-position: left;
}

span.payment-image[name*="afterpay"], span.payment-image[name*="riverty"]  {
  background: url(/static/img/riverty.png) no-repeat;
  background-size: 100%;
  width: 34px;
  display: inline-block;
  background-position: left;
}

span.payment-image[name*="applepay"] {
  background: url(/static/img/applepay.png) no-repeat;
  background-size: 100%;
  width: 29px;
  display: inline-block;
  background-position: left;
  margin-right: 2px;
}

span.payment-image[name*="capayable"] {
  background: url(/static/img/Capayable.png) no-repeat;
  background-size: 100%;
  width: 28px;
  height: 38.217284px;
  display: inline-block;
  background-position: left;
}

span.payment-image[name*="focum"] {
  background: url(/static/img/focum.png) no-repeat;
  background-size: 100%;
  width: 28px;
  height: 22.217284px;
  display: inline-block;
  background-position: left;
}

span.payment-image[name*="klarna"] {
  background: url(/static/img/Klarna.png) no-repeat;
  background-size: 100%;
  width: 38px;
  height: 22.217284px;
  display: inline-block;
  background-position: left;
}

span.payment-image[name*="vvv"] {
  background: url(/static/img/vvv.png) no-repeat;
  background-size: 100%;
  width: 25px;
  height: 26.217284px;
  display: inline-block;
  background-position: left;
}

span.payment-image[name*="webshop"] {
  background: url(/static/img/webshop.jpg) no-repeat;
  background-size: 100%;
  width: 26px;
  height: 26.217284px;
  display: inline-block;
  background-position: left;
}

span.payment-image[name*="invoice:payment"] {
  background: url(/static/img/invoice.jpg) no-repeat;
  background-size: 100%;
  width: 27px;
  height: 30px;
  display: inline-block;
  background-position: left;
}

span.payment-image[name*="amex"] {
    background: url(/static/img/amex.png) no-repeat;
    background-size: 100%;
    width: 35px;
    height: 24px;
    display: inline-block;
    background-position: left;
}

span.payment-image[name*="in3"] {
    background: url(/static/img/in3.png) no-repeat;
    background-size: 100%;
    width: 35px;
    height: 24px;
    display: inline-block;
    background-position: left;
}

span.payment-image[name*="dirdeb"] {
    background: url(/static/img/SEPA-incasso-logo.png) no-repeat;
    background-size: 100%;
    width: 35px;
    height: 24px;
    display: inline-block;
    background-position: left;
}

span.payment-image[name*="payment_opayo:card"] {
  background: url(/static/img/opayo.png) no-repeat;
  background-size: 100%;
  padding: 10px 0 !important;
  width: 150px;
  height: auto;
  display: inline-block;
  background-position: left;
}

span.payment-image[name*="payment_in_store:payment"] {
  background: url(/static/img/pay_in_store.png) no-repeat;
  background-size: 100%;
  width: 35px;
  height: 24px;
  display: inline-block;
  background-position: left;
}

ul#id_payment_method_form-payment_method input {
  position: relative;
  left: -20px;
  margin-right: 20px;
}

.form-group.id_shipping_address_form-line1, .form-group.id_billing_address_form-line1  {
  width: 80%;
  float: left;
}

.form-group.id_shipping_address_form-house_number, .form-group.id_billing_address_form-house_number {
  width: 20%;
  float: left;
} 
.error span {
    color: red;
}
.payment_methods .errorlist {
    color: red;
}

@media only screen and (max-width: 991px) {
  #onstepcheckout select {
    overflow: hidden;
  }
}