# Generated by Django 2.2.13 on 2021-08-18 13:42

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('checkout', '0010_auto_20201117_1212'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='paymentmethod',
            options={'ordering': ['ordering', 'label']},
        ),
        migrations.AddField(
            model_name='paymentmethod',
            name='ordering',
            field=models.IntegerField(default=0, null=True),
        ),
    ]
