{% extends "checkout/partials/formblock.html" %}

{% block panel_body %}
  {% if show_address_chooser %}
    {% for field in address_form.visible_fields %}
      {% include "checkout/partials/address_form_field.html" with field=field %}
    {% endfor %}
  {% endif %} 

  {% for field in form.hidden_fields %}
    {{ field }}
  {% endfor %}

  {% for field in form.visible_fields %}
      {% if field.name != "notes" %}
        {% include "checkout/partials/address_form_field.html" with field=field %}
      {% endif %}
  {% endfor %}
{% endblock %}
