# Generated by Django 2.0.13 on 2019-09-04 12:33

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('checkout', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='paymentmethod',
            name='requires_permission',
            field=models.BooleanField(default=False, help_text="Only users with the 'Allow <name of this payment method>' permission are allowed to use this paymentmethod"),
        ),
    ]
