# Generated by Django 2.2.13 on 2020-11-17 11:12

from django.db import migrations
import oscar.models.fields.autoslugfield


class Migration(migrations.Migration):

    dependencies = [
        ('checkout', '0009_fill_surcharge_code'),
    ]

    operations = [
        migrations.AlterField(
            model_name='surcharge',
            name='code',
            field=oscar.models.fields.autoslugfield.AutoSlugField(blank=True, editable=False, max_length=128, populate_from='name', unique=True, verbose_name='Code'),
        ),
    ]
