{% extends "checkout/partials/address_block.html" %}

{% block panel_footer %}
<div class="well">
  <div class="form-inline">
    {#% select_billing_address_form lives in the global template context %#}
    {% include "partials/form_field.html" with field=select_billing_address_form.require_billing_address %}
  </div>
</div>
{% endblock %}
