{% extends "checkout/partials/shipping_address_notes_block.html" %}
{% load i18n %}

{% block panel_body %}
    {# This assumes that the last field of a shipping_address is the notes field #}
    {% include "oscar/partials/form_field.html" with field=form.visible_fields|last %}
{% endblock %}
