# Generated by Django 4.2.11 on 2024-10-03 06:48

from django.db import migrations, models
from django.conf import settings

class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="DirectPayment",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("unique_identifier", models.CharField(max_length=255, unique=True)),
                (
                    "status",
                    models.CharField(
                        choices=[(status, status) for status in settings.OSCAR_ORDER_STATUS_PIPELINE.keys()],
                        max_length=255,
                    ),
                ),
                ("callback_function", models.CharField(max_length=255)),
            ],
        ),
    ]
