# Generated by Django 2.0.13 on 2019-10-30 14:18

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('checkout', '0002_paymentmethod_requires_permission'),
    ]

    operations = [
        migrations.CreateModel(
            name='Surcharge',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=32)),
                ('excl_tax', models.DecimalField(blank=True, decimal_places=2, max_digits=12, null=True)),
                ('incl_tax', models.DecimalField(blank=True, decimal_places=2, max_digits=12, null=True)),
                ('percentage', models.DecimalField(blank=True, decimal_places=2, max_digits=12, null=True)),
                ('is_percentage', models.BooleanField(default=False)),
            ],
        ),
        migrations.AddField(
            model_name='paymentmethod',
            name='surcharge',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='checkout.Surcharge'),
        ),
    ]