{% extends "checkout/partials/formblock.html" %}
{% load i18n %}

{% block panel_body %}
  {% for field in form.hidden_fields %}
    {{ field }}
  {% endfor %}
  {% include "partials/form_field.html" with field=form.username %}
  {% include "partials/form_field.html" with field=form.options nolabel=True %}
  <div id="passwords">
  {% include "partials/form_field.html" with field=form.password %}
  {% include "partials/form_field.html" with field=password_check_form.password2 %}
  <p id="loginbutton">
      <button class="btn btn-primary pull-right" type="submit">{% trans "Login" %}</button>
      <small><a href="{% url 'password-reset' %}">{% trans "I've forgotten my password" %}</a></small>
  </p>
  </div>
 
{% endblock %}
