# Generated by Django 2.0.13 on 2019-10-31 11:03

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('checkout', '0003_auto_20191030_1418'),
    ]

    operations = [
        migrations.AlterField(
            model_name='paymentmethod',
            name='surcharge',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='checkout.Surcharge'),
        ),
    ]
