{% extends "checkout/partials/formblock.html" %}

{% block panel_body %}
{% for field in form.hidden_fields %}
{{ field }}
{% endfor %}

{% for field in form.visible_fields %}
{% include "oscar/partials/form_field.html" with field=field nolabel=nolabel|default_if_none:False %}
{% endfor %}
{% endblock %}
