# Generated by Django 2.2.13 on 2020-08-18 09:08

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('checkout', '0006_paymentmethod_send_invoice'),
    ]

    operations = [
        migrations.AddField(
            model_name='surcharge',
            name='include_shipping_charges',
            field=models.BooleanField(default=True),
        ),
    ]
