# Generated by Django 2.2.13 on 2020-11-17 11:07

from django.db import migrations


def forward(apps, schema_editor):
    Surcharge = apps.get_model("checkout", "Surcharge")
    for s in Surcharge.objects.all():
        s.save()


class Migration(migrations.Migration):

    dependencies = [
        ('checkout', '0008_surcharge_code'),
    ]

    operations = [
        migrations.RunPython(forward, migrations.RunPython.noop),
    ]
