# Generated by Django 2.0.13 on 2019-12-03 08:45

from decimal import Decimal
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('checkout', '0004_auto_20191031_1103'),
    ]

    operations = [
        migrations.AddField(
            model_name='paymentmethod',
            name='max_amount',
            field=models.DecimalField(blank=True, decimal_places=2, help_text='Some payment methods like pay after have a maximum amount to protect the merchant. If the basket contains more then this amount, the payment method is not available', max_digits=12, null=True, verbose_name='Maximum basket amount'),
        ),
        migrations.AddField(
            model_name='paymentmethod',
            name='min_amount',
            field=models.DecimalField(decimal_places=2, default=Decimal('0.00'), help_text='The minimum amount before this payment is available', max_digits=12, verbose_name='Minimum basket amount'),
        ),
        migrations.AlterField(
            model_name='paymentmethod',
            name='surcharge',
            field=models.ForeignKey(blank=True, null=True, on_delete=models.deletion.CASCADE, to='checkout.Surcharge', verbose_name='Surcharge'),
        ),
    ]
