{% load i18n %}
{% load currency_filters ocyanjson %}
{% ocyanjson "oscar" "show_tax_everywhere" False as show_tax %}

<div class="item">
    <div>
        <a href="{{ product.get_absolute_url }}">
            {{ product.get_title }}
        </a>
        {% with attributes=line.attributes.all %}
            {% if attributes %}
                <a data-toggle="collapse" href="#{{ id }}_lineoptions_{{ forloop.counter }}">
                    <span class="fa-stack pull-right">
                        <i class="fa fa-circle-thin fa-stack-2x"></i>
                        <i class="fa fa-info fa-stack-1x"></i>
                    </span>
                </a>
                <div class="collapse" id="{{ id }}_lineoptions_{{ forloop.counter }}">
                <ul>
                    {% for attr in attributes %}
                        <li>
                            {{ attr.option.name }}
                            <strong>
                                {% if attr.value == 'on' %}{% trans "Yes" %}{% else %}{{ attr.value }}{% endif %}
                            </strong>
                        </li>
                    {% endfor %}
                </ul>
            </div>
        {% endif %}
    {% endwith %}
    </div>
    <span class="price">{{ line.line_price_incl_tax|currency:basket.currency }}</span>
    {% if show_tax %}<br/>
    {% with line.line_price_excl_tax|currency:line.price_currency as price_excl_tax %}
    <small>{% blocktrans %}Excl. tax {{ price_excl_tax }}{% endblocktrans %}</small>
    {% endwith %}{% endif %}
</div>
