# Generated by Django 3.2.13 on 2022-06-07 11:54

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('checkout', '0013_surcharge_data_migration'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='paymentmethod',
            name='surcharge',
        ),
        migrations.DeleteModel(
            name='Surcharge',
        ),
    ]
