{% extends "layout.html" %}
{% load i18n %}

{% block title %}
    {% trans 'Order not found' %} | {{ block.super }}
{% endblock %}
{% block headertext %}
<div class="container">
    {% trans "We couldn't find your order." %}
    <div class="lead">
        {% trans "Sometimes we can't find the order because the order was placed in an app instead of the browser you are now using. When the order is completed, we will notify you by email. If you successfully made payment, but do not receive an email from us in the next 15 minutes, please contact our customer service." %}
    </div>
</div>
{% endblock %}
