# -*- coding: utf-8 -*-

__all__ = ["OcyanOscar3CheckoutPluginConfig"]


def __getattr__(name):
    if name == "OcyanOscar3CheckoutPluginConfig":
        from .entrypoint import OcyanOscar3CheckoutPluginConfig

        return OcyanOscar3CheckoutPluginConfig
    raise AttributeError(f"module {__name__!r} has no attribute {name!r}")
