from django.conf import settings
from django.utils.translation import gettext_lazy as _

from oscar.apps.payment.forms import BillingAddressForm as BaseBillingAddressForm


class BillingAddressForm(BaseBillingAddressForm):
    phone_number_fields = {}

    field_order = settings.OSCAR_BILLING_FORM_FIELD_ORDER

    class Meta(BaseBillingAddressForm.Meta):
        fields = settings.OSCAR_BILLING_FORM_FIELDS
        help_texts = {"email": _("The invoice will be sent to this e-mail address")}
