# Generated by Django 2.2.9 on 2020-04-22 11:49

from django.db import migrations, models

from ocyan.core.fender import config

class Migration(migrations.Migration):

    dependencies = [
        ('checkout', '0005_payment_method_limits'),
    ]

    operations = [
        migrations.AddField(
            model_name='paymentmethod',
            name='send_invoice',
            field=models.BooleanField(default=config.get("oscar_communications", "invoice", False)),
        ),
    ]
