{% load i18n wagtailcore_tags %}
{% trans "I agree to the terms of service" %}.{% if settings.ocyan_plugin_wagtail.OcyanSettings.terms %} <a href="{% pageurl settings.ocyan_plugin_wagtail.OcyanSettings.terms %}" target="_blank">{% trans 'Click here for more info' %}</a>{% endif %} <span>*</span>