# Generated by Django 3.2.13 on 2022-06-07 11:21

from django.db import migrations, models
import oscar.models.fields.autoslugfield


class Migration(migrations.Migration):

    dependencies = [
        ('address', '0001_initial'),
        ('checkout', '0011_auto_20210818_1542'),
        ('address', '0008_useraddress_email'),
    ]

    operations = [
        migrations.CreateModel(
            name='FixedSurcharge',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=32)),
                ('code', oscar.models.fields.autoslugfield.AutoSlugField(blank=True, editable=False, max_length=128, populate_from='name', unique=True, verbose_name='Code')),
                ('excl_tax', models.DecimalField(decimal_places=2, max_digits=12)),
                ('incl_tax', models.DecimalField(decimal_places=2, max_digits=12)),
                ('country', models.ManyToManyField(blank=True, to='address.Country', verbose_name='Countries')),
                ('payment_method', models.ManyToManyField(blank=True, to='checkout.PaymentMethod', verbose_name='Payment methods')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='PercentageSurcharge',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=32)),
                ('code', oscar.models.fields.autoslugfield.AutoSlugField(blank=True, editable=False, max_length=128, populate_from='name', unique=True, verbose_name='Code')),
                ('percentage', models.DecimalField(decimal_places=2, max_digits=12)),
                ('include_shipping_charges', models.BooleanField(default=True)),
                ('country', models.ManyToManyField(blank=True, to='address.Country', verbose_name='Countries')),
                ('payment_method', models.ManyToManyField(blank=True, to='checkout.PaymentMethod', verbose_name='Payment methods')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]
