from django.forms import fields, widgets


class StandardsCompliantRadioWidget(widgets.RadioSelect):
    option_template_name = "checkout/widgets/radio_options.html"


class MethodsChoiceField(fields.ChoiceField):
    def __init__(self, *, choices=(), methods=(), **kwargs):
        super().__init__(**kwargs)
        self.choices = choices
        self.methods = methods
