# Generated by Django 3.2.19 on 2023-07-10 10:36

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('checkout', '0016_surcharge_flat_rate_price_migration'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='fixedsurcharge',
            name='excl_tax',
        ),
        migrations.RemoveField(
            model_name='fixedsurcharge',
            name='incl_tax',
        ),
    ]
