{% extends "layout.html" %}

{% load i18n %}

{% block title %}
    {% trans "Cancelled" %}
{% endblock %}

{% block headertext %}
    {% trans "Cancelled" %}
{% endblock %}

{% block content_wrapper %}
<section>
    <div class="container">
        <p>{% blocktrans %}De order {{ order }} is geannuleerd. {% endblocktrans %}<a href="/">{% trans "Terug naar home." %}</a></p>
    </div>
</section>
{% endblock %}
