{% extends "checkout/partials/formblock.html" %}
{% load i18n ocyan %}

{% block panel_header %}
  <label class="control-label required{% if form.errors %} error{% endif %}">
      {% trans 'Choose a shippingmethod' %}
  </label>
{% endblock %}

{% block panel_body %}
  {% if form.hidden_fields %}
    {{ form }}
  {% else %}
    <ul id="shipping-method-list">
    {% for widget, method in form.method_code.subwidgets|zip:form.method_code.field.methods %}
      <li>
        {{ widget }}
        <span class="shipping-price">
          {% method_price method as session %}
          {% include 'oscar/catalogue/partials/price.html' %}
        </span>
      </li>
    {% endfor %}
    {% if form.method_code.errors %}
      <li><span class="error-block"><i class="icon-exclamation-sign"></i> {{ form.method_code.errors|join:" " }}</span></li>
    {% endif %}
    </ul>
  {% endif %}
{% endblock %}


<ul id="id_shipping_method_form-method_code_0">
    
  <li>
    <label for="id_shipping_method_form-method_code_0">
      <input type="radio" name="shipping_method_form-method_code" value="free-shipping" id="id_shipping_method_form-method_code_0" required="">
      Free shipping
    </label>
    <span class="shipping-price">€0.00</span>
  </li>
    
  <li>
    <label for="id_shipping_method_form-method_code_1">
      <input type="radio" name="shipping_method_form-method_code" value="postnel" id="id_shipping_method_form-method_code_1" required="">
      Postnel
    </label>
    <span class="shipping-price">€10.00</span>
  </li>
    
</ul>