# Generated by Django 3.2.19 on 2023-07-10 10:29

from django.db import migrations, models

from oscar.core.loading import get_classes

get_default_tax_rate, ALL_TAX_RATES = get_classes(
    "taxes.taxrate",
    [
        "get_default_tax_rate",
        "ALL_TAX_RATES",
    ],
)


class Migration(migrations.Migration):

    dependencies = [
        ('checkout', '0014_auto_20220607_1354'),
    ]

    operations = [
        migrations.AddField(
            model_name='fixedsurcharge',
            name='price',
            field=models.DecimalField(decimal_places=2, default=0, max_digits=12),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='fixedsurcharge',
            name='vat_rate',
            field=models.CharField(choices=ALL_TAX_RATES, default=get_default_tax_rate(), max_length=5, null=True, verbose_name='VAT rate'),
        ),
        migrations.AddField(
            model_name='percentagesurcharge',
            name='vat_rate',
            field=models.CharField(choices=ALL_TAX_RATES, default=get_default_tax_rate(), max_length=5, null=True, verbose_name='VAT rate'),
        ),
    ]
